/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose;

import java.util.Properties;

import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.log.DefaultLog;
import be.ac.vub.cocompose.log.Log;

/**
 * Main CoCompose class for stand-alone use and settings registry.
 * @author Dennis Wagelaar
 * @version 0.2.8, 19/04/2004
 * @since 0.2.8
 */
public class CoCompose
{
  private static Properties defaults = new Properties();
  static
  {
    defaults.setProperty(
      "model.factory", "be.ac.vub.cocompose.lang.impl.DefaultModelFactory");
    defaults.setProperty(
      "model.writer",  "be.ac.vub.cocompose.io.xmi.XMIModelWriter");
    defaults.setProperty(
      "model.reader",  "be.ac.vub.cocompose.io.xmi.XMIModelReader");

    defaults.setProperty(
      "debugging.enabled", "false");

    defaults.setProperty(
      "logging.log", "be.ac.vub.cocompose.log.DefaultLog");
  }
  private static CoCompose Singleton = new CoCompose();
  
  private Properties settings = null;
  private Log log = null;
  private ModelFactory factory = null;

  /**
   * Runs a stand-alone CoCompose.
   * @param args command-line arguments
   */
  public static void main(String[] args)
  {
    // TODO stub
    System.out.println("Stand-alone CoCompose2 not yet implemented...");
  }

  /**
   * Gets the singleton CoCompose instance.
   * @return the singleton CoCompose instance.
   */
  public static CoCompose getDefault()
  {
    return Singleton;
  }
  
  /**
   * Private constructor for Singleton pattern.
   */
  private CoCompose()
  {
    defaults.setProperty(
      "model.factory", "be.ac.vub.cocompose.lang.impl.DefaultModelFactory");
    defaults.setProperty(
      "model.writer",  "be.ac.vub.cocompose.io.xmi.XMIModelWriter");
    defaults.setProperty(
      "model.reader",  "be.ac.vub.cocompose.io.xmi.XMIModelReader");

    defaults.setProperty(
      "debugging.enabled", "false");

    defaults.setProperty(
      "logging.log", "be.ac.vub.cocompose.log.DefaultLog");
    settings = new Properties(defaults);
    initSettings();
  }

  /**
   * Returns the standard logger.
   * @return the standard logger.
   */
  public Log getLog()
  {
    return log;
  }

  /**
   * Gets the model element factory
   * @return the factory
   */
  public ModelFactory getFactory()
  {
    return factory;
  }

  /**
   * Gets a global CoCompose setting
   * @param name the setting name
   * @return the setting value
   */
  public String getSetting(String name)
  {
    return settings.getProperty(name);
  }

  /**
   * Gets the global CoCompose settings.
   * @return the settings.
   */
  public Properties getSettings()
  {
    return settings;
  }

  /**
   * Sets the global CoCompose settings.
   * @param properties the settings.
   */
  public void setSettings(Properties properties)
  {
    settings = properties;
    initSettings();
  }

  /**
   * (Re-)initialises this class after settings have been set.
   */
  private void initSettings()
  {
    try
    {
      log = (Log) Class.forName(
        getSetting("logging.log")).newInstance();
      log.setDebugging(
        settings.getProperty("debugging.enabled").equals("true"));
      factory = (ModelFactory) Class.forName(
        getSetting("model.factory")).newInstance();
    }
    catch (Exception e)
    {
      Exception we = new Exception(
        "Error: could not initialise CoCompose settings: " + e.getMessage(), 
        e);
      if (log == null)
        log = new DefaultLog();
      log.report(we);
    }
  }
}
