/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposeOutlinePage.java,v 1.3 2004/04/27 16:43:58 dwagelaa Exp $
 * $Log: CoComposeOutlinePage.java,v $
 * Revision 1.3  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.eclipse;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.part.PageBook;

import be.ac.vub.cocompose.eclipse.dnd.CoComposeTemplateTransferDropTargetListener;
import be.ac.vub.cocompose.eclipse.editparts.TreePartFactory;

/**
 * Tree outline page for CoCompose elements.
 * @author Dennis Wagelaar
 * @version 0.2.9, 29/04/2004
 * @since 0.2.0
 */
public class CoComposeOutlinePage
  extends ContentOutlinePage
  implements IAdaptable
{
  protected static final int ID_OUTLINE  = 0;
  protected static final int ID_OVERVIEW = 1;

  private CoComposeEditor editor;
  private IAction showOutlineAction;
  private IAction showOverviewAction;
  private PageBook pageBook;
  private Control outline;
  private Canvas overview;
  private boolean overviewInitialized = false;
  private Thumbnail thumbnail;

  /**
   * Constructs a CoComposeOutlinePage
   * @param viewer
   */
  public CoComposeOutlinePage(EditPartViewer viewer, CoComposeEditor editor)
  {
    super(viewer);
    this.editor = editor;
  }

  /**
   * @see org.eclipse.ui.part.IPageBookViewPage#init(org.eclipse.ui.part.IPageSite)
   */
  public void init(IPageSite pageSite) 
  {
    super.init(pageSite);
    ActionRegistry registry = editor.getActionRegistry();
    IActionBars bars = pageSite.getActionBars();
    String id = IWorkbenchActionConstants.UNDO;
    bars.setGlobalActionHandler(id, registry.getAction(id));
    id = IWorkbenchActionConstants.REDO;
    bars.setGlobalActionHandler(id, registry.getAction(id));
    id = IWorkbenchActionConstants.DELETE;
    bars.setGlobalActionHandler(id, registry.getAction(id));
    bars.updateActionBars();
  }

  /**
   * Configures the outline viewer. 
   */
  protected void configureOutlineViewer()
  {
    getViewer().setEditDomain(editor.getEditDomain());
    getViewer().setEditPartFactory(new TreePartFactory());
    ContextMenuProvider provider = new CoComposeContextMenuProvider(getViewer(), editor.getActionRegistry());
    getViewer().setContextMenu(provider);
    getSite().registerContextMenu(
      "be.ac.vub.cocompose.outline.contextmenu", //$NON-NLS-1$
      provider, getSite().getSelectionProvider());
    getViewer().setKeyHandler(editor.getCommonKeyHandler());
    getViewer().addDropTargetListener(
      new CoComposeTemplateTransferDropTargetListener(getViewer()));
    IToolBarManager tbm = getSite().getActionBars().getToolBarManager();
    showOutlineAction = new Action() {
      public void run() {
        showPage(ID_OUTLINE);
      }
    };
    showOutlineAction.setImageDescriptor(CoComposeImages.OUTLINE_OBJ);
    tbm.add(showOutlineAction);
    showOverviewAction = new Action() {
      public void run() {
        showPage(ID_OVERVIEW);
      }
    };
    showOverviewAction.setImageDescriptor(CoComposeImages.OVERVIEW_OBJ);
    tbm.add(showOverviewAction);
    showPage(ID_OUTLINE);
  }

  /**
   * @see org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite parent)
  {
    pageBook = new PageBook(parent, SWT.NONE);
    outline = getViewer().createControl(pageBook);
    overview = new Canvas(pageBook, SWT.NONE);
    pageBook.showPage(outline);
    configureOutlineViewer();
    hookOutlineViewer();
    initializeOutlineViewer();
  }
  
  /**
   * @see org.eclipse.ui.part.IPage#dispose()
   */
  public void dispose()
  {
    unhookOutlineViewer();
    super.dispose();
  }
  
  /**
   * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
   */
  public Object getAdapter(Class type) 
  {
    if (type == ZoomManager.class)
      return ((ScalableFreeformRootEditPart) editor.getGraphicalViewer()
            .getRootEditPart()).getZoomManager();
    return null;
  }

  /**
   * @see org.eclipse.ui.part.IPage#getControl()
   */
  public Control getControl() 
  {
    return pageBook;
  }

  /**
   * Hooks the outline viewer into the page.
   */
  protected void hookOutlineViewer()
  {
    editor.getSelectionSynchronizer().addViewer(getViewer());
  }

  /**
   * Initializes the outline viewer with contents.
   */
  protected void initializeOutlineViewer()
  {
    getViewer().setContents(editor.getModel());
  }
  
  /**
   * Initializes the overview.
   */
  protected void initializeOverview() 
  {
    LightweightSystem lws = new LightweightSystem(overview);
    RootEditPart rep = editor.getGraphicalViewer().getRootEditPart();
    if (rep instanceof ScalableFreeformRootEditPart) 
    {
      ScalableFreeformRootEditPart root = (ScalableFreeformRootEditPart)rep;
      thumbnail = new ScrollableThumbnail((Viewport) root.getFigure());
      thumbnail.setSource(root.getLayer(LayerConstants.PRINTABLE_LAYERS));
      lws.setContents(thumbnail);
    }
  }
  
  /**
   * Show the page.
   * @param id ID_OUTLINE or ID_OVERVIEW.
   */
  protected void showPage(int id) 
  {
    if (id == ID_OUTLINE) 
    {
      showOutlineAction.setChecked(true);
      showOverviewAction.setChecked(false);
      pageBook.showPage(outline);
      if (thumbnail != null)
        thumbnail.setVisible(false);
    } 
    else if (id == ID_OVERVIEW) 
    {
      if (!overviewInitialized)
        initializeOverview();
      showOutlineAction.setChecked(false);
      showOverviewAction.setChecked(true);
      pageBook.showPage(overview);
      thumbnail.setVisible(true);
    }
  }
  
  /**
   * Unhooks the outline viewer from the page.
   */
  protected void unhookOutlineViewer()
  {
    editor.getSelectionSynchronizer().removeViewer(getViewer());
  }
}
