/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposePrintAction.java,v 1.4 2004/04/28 16:30:08 dwagelaa Exp $
 * $Log: CoComposePrintAction.java,v $
 * Revision 1.4  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 */

package be.ac.vub.cocompose.eclipse.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.print.PrintGraphicalViewerOperation;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.printing.PrintDialog;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.draw2d.EPSGraphics;
import be.ac.vub.cocompose.eclipse.editparts.GraphicalPartFactory;
import be.ac.vub.cocompose.io.ModelReader;
import be.ac.vub.cocompose.lang.core.Model;

/**
 * CoCompose Eclipse printing
 * @author Dennis Wagelaar
 * @version 0.2.9, 28/04/2004
 * @since 0.2.8
 */
public class CoComposePrintAction
  extends Action
  implements IObjectActionDelegate
{
  private Object contents;
  private IFile selectedFile;

  /**
   * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
   */
  public void setActivePart(IAction action, IWorkbenchPart targetPart)
  {
  }

  /**
   * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
   */
  public void run(IAction action)
  {
    Shell shell = new Shell();
    GraphicalViewer viewer = new ScrollingGraphicalViewer();
    viewer.createControl(shell);
    viewer.setEditDomain(new DefaultEditDomain(null));
    viewer.setRootEditPart(new ScalableFreeformRootEditPart());
    viewer.setEditPartFactory(new GraphicalPartFactory());
    viewer.setContents(getContents());
    viewer.flush();
    
    int printMode = new PrintModeDialog(shell).open();
    if (printMode == -1)
      return;
    PrintDialog dialog = new PrintDialog(shell, SWT.NULL);
    PrinterData data = dialog.open();
    if (data != null) {
      PrintGraphicalViewerOperation op = 
        new PrintGraphicalViewerOperation(new Printer(data), viewer);
      op.setPrintMode(printMode);
      op.run(selectedFile.getName());
    }
  }

  /**
   * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
   */
  public void selectionChanged(IAction action, ISelection selection)
  {
    if (!(selection instanceof IStructuredSelection))
      return;
    IStructuredSelection sel = (IStructuredSelection) selection;
    if (sel.size() != 1)
      return;
    selectedFile = (IFile) sel.getFirstElement();
    try 
    {
      ModelReader reader = (ModelReader) Class.forName(
        CoCompose.getDefault().getSetting("model.reader")).newInstance();
      Model model = reader.read(selectedFile.getContents(false), 
        CoCompose.getDefault().getFactory());
      model.setUri(selectedFile.getLocation().toString());
      setContents(model);
    }
    catch (Exception e) 
    {
      CoCompose.getDefault().getLog().report(e);
    }
  }

  /**
   * Gets printing contents.
   * @return printing contents.
   */
  protected Object getContents() 
  {
    return contents;
  }

  /**
   * Sets printing contents.
   * @param o printing contents.
   */
  protected void setContents(Object o) 
  {
    contents = o;
  }
}
