/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.commands;

import org.eclipse.gef.commands.Command;

import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.operations.DeleteElement;

/**
 * Delete command for CoCompose elements.
 * @author Dennis Wagelaar
 * @version 0.2.0, 02/12/2003
 * @since 0.2.0
 */
public class DeleteCommand extends Command
{
  private DeleteElement deleteElement = new DeleteElement();
  private ModelElement element = null;

  /**
   * Creates a DeleteCommand. 
   */
  public DeleteCommand()
  {
    super("Delete");
  }

  /**
   * @see org.eclipse.gef.commands.Command#execute()
   */
  public void execute() 
  {
    primExecute();
  }

  /**
   * @see org.eclipse.gef.commands.Command#redo()
   */
  public void redo() 
  {
    primExecute();
  }

  /**
   * @see org.eclipse.gef.commands.Command#undo()
   */
  public void undo() 
  {
    try
    {
      deleteElement.undo();
    }
    catch (ModelElementException e)
    {
      // should never occur
      e.printStackTrace();
    }
  }

  /**
   * Gets the element to be deleted.
   * @return the element to be deleted.
   */
  public ModelElement getElement()
  {
    return element;
  }

  /**
   * Sets the element to be deleted.
   * @param element - the element to be deleted.
   */
  public void setElement(ModelElement element)
  {
    this.element = element;
  }

  /**
   * Execute implementation.
   * @see org.eclipse.gef.commands.Command#execute()
   */
  protected void primExecute() 
  {
    try
    {
      deleteElement.visit(getElement());
    }
    catch (ModelElementException e)
    { // should never occur
      e.printStackTrace();
    }
  }
}
