/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: TextTransferDropTargetListener.java,v 1.1 2004/04/27 16:43:58 dwagelaa Exp $
 * $Log: TextTransferDropTargetListener.java,v $
 * Revision 1.1  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.eclipse.dnd;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.dnd.AbstractTransferDropTargetListener;
import org.eclipse.swt.dnd.Transfer;

/**
 * This listener handles text that is dropped onto
 * the CoCompose editor.
 * @author Dennis Wagelaar
 * @version 0.2.9, 27/04/2004
 * @since 0.2.9
 */
public class TextTransferDropTargetListener
  extends AbstractTransferDropTargetListener
{
  /**
   * Creates a TextTransferDropTargetListener.
   * @param viewer the CoCompose model viewer.
   * @param xfer
   */
  public TextTransferDropTargetListener(EditPartViewer viewer, Transfer xfer)
  {
    super(viewer, xfer);
  }

  /**
   * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#updateTargetRequest()
   */
  protected void updateTargetRequest()
  {
    getNativeDropRequest().setData(getCurrentEvent().data);
  }

  /**
   * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#createTargetRequest()
   */
  protected Request createTargetRequest() 
  {
    return new NativeDropRequest();
  }

  /**
   * Gets the target request as a drop request.
   * @return the drop request.
   */
  protected NativeDropRequest getNativeDropRequest() 
  {
    return (NativeDropRequest) getTargetRequest();
  }
}
