/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: EPSGraphics.java,v 1.3 2004/04/28 16:30:08 dwagelaa Exp $
 * $Log: EPSGraphics.java,v $
 * Revision 1.3  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.2  2004/04/26 16:30:43  dwagelaa
 * Added CVS tags
 *
 */

package be.ac.vub.cocompose.eclipse.draw2d;

import java.awt.BasicStroke;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Stack;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;
import org.jibble.epsgraphics.EpsGraphics2D;

/**
 * Graphics class for creating EPS images. 
 * @author Dennis Wagelaar
 * @version 0.2.9, 28/04/2004
 * @since 0.2.9
 */
public class EPSGraphics extends Graphics
{
  private static final float[] DASH        = {5};
  private static final float[] DOT         = {1};
  private static final float[] DASHDOT     = {5, 1};
  private static final float[] DASHDOTDOT  = {5, 1, 1};
  
  private static final boolean FILL_MODE = true;
  private static final boolean DRAW_MODE = true;
  
  protected class State
  {
    private double zoom;
    private int x;
    private int y;
    private int lineWidth;
    private Shape clip;
    
    /**
     * Creates a new State.
     * @param zoom the zoom factor
     * @param x the x offset
     * @param y the y offset
     * @param lineWidth the line width
     * @param clip the clip region
     */
    public State(double zoom, int x, int y, int lineWidth, Shape clip)
    {
      this.zoom = zoom;
      this.x = x;
      this.y = y;
      this.lineWidth = lineWidth;
      this.clip = clip;
    }
  }
  
  private EpsGraphics2D inner;
  private GC gc = new GC(Display.getDefault());
  private Stack stateStack = new Stack();
  private double zoom = 1;
  private int x = 0;
  private int y = 0;

  /**
   * Creates an EPSGraphics.
   * @param title the EPS document title.
   */
  public EPSGraphics(String title)
  {
    super();
    inner = new EpsGraphics2D(title);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#clipRect(org.eclipse.draw2d.geometry.Rectangle)
   */
  public void clipRect(Rectangle r)
  {
    inner.clipRect(r.x, r.y, r.width, r.height);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#dispose()
   */
  public void dispose()
  {
    inner.dispose();
    gc.dispose();
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawArc(int, int, int, int, int, int)
   */
  public void drawArc(int x, int y, int w, int h, int offset, int length)
  {
    inner.drawArc(x, y, w, h, offset, length);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillArc(int, int, int, int, int, int)
   */
  public void fillArc(int x, int y, int w, int h, int offset, int length)
  {
    inner.fillArc(x, y, w, h, offset, length);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillGradient(int, int, int, int, boolean)
   */
  public void fillGradient(int x, int y, int w, int h, boolean vertical)
  { // closest match
    inner.fill3DRect(x, y, w, h, vertical);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawFocus(int, int, int, int)
   */
  public void drawFocus(int x, int y, int w, int h)
  { // closest match
    inner.drawRect(x, y, w, h);
  }

  /**
   * Not yet implemented.
   * @see org.eclipse.draw2d.Graphics#drawImage(org.eclipse.swt.graphics.Image, int, int)
   */
  public void drawImage(Image srcImage, int x, int y)
  {
    // TODO stub
    //inner.drawImage(toAWTImage(srcImage), x, y, null);
  }

  /**
   * Not yet implemented.
   * @see org.eclipse.draw2d.Graphics#drawImage(org.eclipse.swt.graphics.Image, int, int, int, int, int, int, int, int)
   */
  public void drawImage(
    Image srcImage,
    int x1,
    int y1,
    int w1,
    int h1,
    int x2,
    int y2,
    int w2,
    int h2)
  {
    // TODO stub
    //inner.drawImage(toAWTImage(srcImage), x2, y2, x2+w2, y2+h2, x1, y1, x1+w1, y1+h1, null);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawLine(int, int, int, int)
   */
  public void drawLine(int x1, int y1, int x2, int y2)
  {
    inner.drawLine(x1, y1, x2, y2);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawOval(int, int, int, int)
   */
  public void drawOval(int x, int y, int w, int h)
  {
    inner.drawOval(x, y, w, h);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillOval(int, int, int, int)
   */
  public void fillOval(int x, int y, int w, int h)
  {
    inner.fillOval(x, y, w, h);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawPolygon(org.eclipse.draw2d.geometry.PointList)
   */
  public void drawPolygon(PointList points)
  {
    inner.drawPolygon(toPolygon(points));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillPolygon(org.eclipse.draw2d.geometry.PointList)
   */
  public void fillPolygon(PointList points)
  {
    inner.fillPolygon(toPolygon(points));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawPolyline(org.eclipse.draw2d.geometry.PointList)
   */
  public void drawPolyline(PointList points)
  {
    int nPoints = points.size();
    int[] pts = points.toIntArray();
    int[] xPoints = new int[nPoints];
    int[] yPoints = new int[nPoints];
    for (int i = 0; i < nPoints; i++)
    {
      xPoints[i] = pts[i*2];
      yPoints[i] = pts[i*2+1];
    }
    inner.drawPolyline(xPoints, yPoints, nPoints);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawRectangle(int, int, int, int)
   */
  public void drawRectangle(int x, int y, int width, int height)
  {
    inner.drawRect(x, y, width, height);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillRectangle(int, int, int, int)
   */
  public void fillRectangle(int x, int y, int width, int height)
  {
    inner.fillRect(x, y, width, height);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawRoundRectangle(org.eclipse.draw2d.geometry.Rectangle, int, int)
   */
  public void drawRoundRectangle(Rectangle r, int arcWidth, int arcHeight)
  {
    inner.drawRoundRect(r.x, r.y, r.width, r.height, arcWidth, arcHeight);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillRoundRectangle(org.eclipse.draw2d.geometry.Rectangle, int, int)
   */
  public void fillRoundRectangle(Rectangle r, int arcWidth, int arcHeight)
  {
    inner.fillRoundRect(r.x, r.y, r.width, r.height, arcWidth, arcHeight);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawText(java.lang.String, int, int)
   */
  public void drawText(String s, int x, int y)
  {
    // always expands special characters
    inner.drawString(s, x, transposeTextY(y));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#drawString(java.lang.String, int, int)
   */
  public void drawString(String s, int x, int y)
  {
    // always expands special characters
    inner.drawString(s, x, transposeTextY(y));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillString(java.lang.String, int, int)
   */
  public void fillString(String s, int x, int y)
  {
    // closest match
    // always expands special characters
    inner.drawString(s, x, transposeTextY(y));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#fillText(java.lang.String, int, int)
   */
  public void fillText(String s, int x, int y)
  {
    // closest match
    // always expands special characters
    inner.drawString(s, x, transposeTextY(y));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getBackgroundColor()
   */
  public Color getBackgroundColor()
  {
    return toSWTColor((java.awt.Color) inner.getPaint());
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getClip(org.eclipse.draw2d.geometry.Rectangle)
   */
  public Rectangle getClip(Rectangle rect)
  {
    java.awt.Rectangle clipped = inner.getClipBounds(
      new java.awt.Rectangle(rect.x, rect.y, rect.width, rect.height));
    return new Rectangle(clipped.x, clipped.y, clipped.width, clipped.height);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getFont()
   */
  public Font getFont()
  {
    return gc.getFont();
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getFontMetrics()
   */
  public FontMetrics getFontMetrics()
  {
    return gc.getFontMetrics();
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getForegroundColor()
   */
  public Color getForegroundColor()
  {
    return toSWTColor(inner.getColor());
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getLineStyle()
   */
  public int getLineStyle()
  {
    Stroke stroke = inner.getStroke();
    if (stroke instanceof BasicStroke)
    {
      BasicStroke bs = (BasicStroke) stroke;
      float[] dash = bs.getDashArray();
      if (dash != null)
      {
        if (dash.equals(DASH))
          return SWT.LINE_DASH;
        if (dash.equals(DOT))
          return SWT.LINE_DOT;
        if (dash.equals(DASHDOT))
          return SWT.LINE_DASHDOT;
        if (dash.equals(DASHDOTDOT))
          return SWT.LINE_DASHDOTDOT;
      }
    }
    return SWT.LINE_SOLID;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#getLineWidth()
   */
  public int getLineWidth()
  {
    Stroke stroke = inner.getStroke();
    if (stroke instanceof BasicStroke)
    {
      BasicStroke bs = (BasicStroke) stroke;
      return Math.round(bs.getLineWidth());
    }
    return 0;
  }

  /**
   * Not implemented.
   * @see org.eclipse.draw2d.Graphics#getXORMode()
   */
  public boolean getXORMode()
  {
    // ignored
    return false;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#popState()
   */
  public void popState()
  {
    if (stateStack.size() == 0)
      return;
    State state = (State) stateStack.pop();
    scale(state.zoom / zoom);
    translate(state.x - x, state.y - y);
    setLineWidth(state.lineWidth);
    inner.setClip(state.clip);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#pushState()
   */
  public void pushState()
  {
    State state = new State(zoom, x, y, getLineWidth(), inner.getClip());
    stateStack.push(state);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#restoreState()
   */
  public void restoreState()
  {
    popState();
  }

  /**
   * @see org.eclipse.draw2d.Graphics#scale(double)
   */
  public void scale(double amount)
  {
    inner.scale(amount, amount);
    zoom *= amount;
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setBackgroundColor(org.eclipse.swt.graphics.Color)
   */
  public void setBackgroundColor(Color rgb)
  {
    inner.setPaint(toAWTColor(rgb));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setClip(org.eclipse.draw2d.geometry.Rectangle)
   */
  public void setClip(Rectangle r)
  {
    inner.setClip(r.x, r.y, r.width, r.height);
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setFont(org.eclipse.swt.graphics.Font)
   */
  public void setFont(Font f)
  {
    gc.setFont(f);
//    FontData[] fd = f.getFontData();
//    inner.setFont(
//      new java.awt.Font(fd[0].getName(), fd[0].getStyle(), fd[0].getHeight()));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setForegroundColor(org.eclipse.swt.graphics.Color)
   */
  public void setForegroundColor(Color rgb)
  {
    inner.setColor(toAWTColor(rgb));
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setLineStyle(int)
   */
  public void setLineStyle(int style)
  {
    Stroke stroke = inner.getStroke();
    if (stroke instanceof BasicStroke)
    {
      BasicStroke bs = (BasicStroke) stroke;
      BasicStroke nbs;
      float[] dash = null; // SOLID
      if (SWT.LINE_DASH == style)
        dash = DASH;
      else if (SWT.LINE_DOT == style)
        dash = DOT;
      else if (SWT.LINE_DASHDOT == style)
        dash = DASHDOT;
      else if (SWT.LINE_DASHDOTDOT == style)
        dash = DASHDOTDOT;
      nbs =
        new BasicStroke(
          bs.getLineWidth(),
          bs.getEndCap(),
          bs.getLineJoin(),
          bs.getMiterLimit(),
          dash,
          bs.getDashPhase());
      inner.setStroke(nbs);
    }
  }

  /**
   * @see org.eclipse.draw2d.Graphics#setLineWidth(int)
   */
  public void setLineWidth(int width)
  {
    Stroke stroke = inner.getStroke();
    if (stroke instanceof BasicStroke)
    {
      BasicStroke bs = (BasicStroke) stroke;
      BasicStroke nbs =
        new BasicStroke(
          width,
          bs.getEndCap(),
          bs.getLineJoin(),
          bs.getMiterLimit(),
          bs.getDashArray(),
          bs.getDashPhase());
      inner.setStroke(nbs);
    }
  }

  /**
   * Not implemented.
   * @see org.eclipse.draw2d.Graphics#setXORMode(boolean)
   */
  public void setXORMode(boolean b)
  {
    // ignored
  }

  /**
   * @see org.eclipse.draw2d.Graphics#translate(int, int)
   */
  public void translate(int dx, int dy)
  {
    inner.translate(dx, dy);
    x += dx;
    y += dy;
  }
  
  /**
   * Returns the EPS content of this Graphics object.
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    return inner.toString();
  }

  /**
   * Converts a PointList to a Polygon.
   * @param points the point list to convert.
   * @return the polygon representation of the point list.
   */
  protected Polygon toPolygon(PointList points)
  {
    Polygon pol = new Polygon();
    for (int i = 0; i < points.size(); i++)
    {
      Point pt = points.getPoint(i);
      pol.addPoint(pt.x, pt.y);
    }
    return pol;
  }
  
  /**
   * Converts an Eclipse/SWT Image to an AWT Image.
   * @param srcImage the source SWT image.
   * @return the converted AWT image.
   */
  protected BufferedImage toAWTImage(Image srcImage)
  {
    ImageData src = srcImage.getImageData();
//    byte[] buffer = new byte[src.data.length];
//    System.arraycopy(src.data, 0, buffer, 0, src.data.length);
//    DataBufferByte db = new DataBufferByte(buffer, buffer.length);
//    SampleModel sm;
//    if (src.palette.isDirect && src.depth > 8)
//    {
//      sm =
//        new PixelInterleavedSample(
//          db.getDataType(),
//          src.width,
//          src.height,
//          src.depth / 8,
//          src.bytesPerLine,
//          [src.palette.redShift,
//          src.palette.greenShift,
//          src.palette.blueShift]); 
//    }
//    else
//    {
//      sm =
//        new SinglePixelPackedSampleModel(
//          db.getDataType(),
//          src.width,
//          src.height,
//          src.bytesPerLine,
//          [src.palette.redMask,
//          src.palette.greenMask,
//          src.palette.blueMask]);
//    }

    int type = BufferedImage.TYPE_CUSTOM;
    if (src.palette.isDirect)
    {
      switch (src.depth)
      {
        case 32: type = BufferedImage.TYPE_4BYTE_ABGR; break;
        case 24: type = BufferedImage.TYPE_3BYTE_BGR; break;
        //case 16: type = BufferedImage.TYPE_USHORT_565_RGB; break;
        case 8:  type = BufferedImage.TYPE_BYTE_GRAY; break;
      }
    }
    else
    {
      switch (src.depth)
      {
        case 8:  type = BufferedImage.TYPE_BYTE_INDEXED; break;
      }
    }
    BufferedImage awtImage = new BufferedImage(src.width, src.height, type);
    WritableRaster raster = (WritableRaster) awtImage.getData();
    
    int[] scanline = new int[src.width];
    for (int y = src.y; y < src.height; y++)
    {
      src.getPixels(src.x, y, src.width, scanline, src.x);
      raster.setPixels(src.x, y, src.width, 1, scanline);
    }
    
    return awtImage;
  }
  
  /**
   * Converts an SWT Color to an AWT Color.
   * @param rgb the SWT Color.
   * @return the AWT Color.
   */
  protected java.awt.Color toAWTColor(Color rgb)
  {
    return new java.awt.Color(rgb.getRed(), rgb.getGreen(), rgb.getBlue());
  }
  
  /**
   * Converts an AWT Color to an SWT Color.
   * @param rgb the AWT Color.
   * @return the SWT Color.
   */
  protected Color toSWTColor(java.awt.Color rgb)
  {
    return new Color(Display.getDefault(), rgb.getRed(), rgb.getGreen(), rgb.getBlue());
  }
  
  /**
   * Transpose y coordinate by font height.
   * @param y the y coordinate to transpose.
   * @return the transposed y coordinate.
   */
  protected int transposeTextY(int y)
  {
    return y + getFontMetrics().getAscent();
  }
}
