/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: RefinedElementEditPart.java,v 1.2 2004/04/26 17:39:41 dwagelaa Exp $
 * $Log: RefinedElementEditPart.java,v $
 * Revision 1.2  2004/04/26 17:39:41  dwagelaa
 * Changed the defaultConcept property into the defaultElement property, which is a RefinedElement.
 *
 */

package be.ac.vub.cocompose.eclipse.editparts;

import java.beans.PropertyChangeEvent;
import java.io.File;
import java.net.URL;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import be.ac.vub.cocompose.eclipse.editpolicies.CoComposeXYLayoutEditPolicy;
import be.ac.vub.cocompose.eclipse.figures.ConceptFigure;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.RefinedElement;

/**
 * Eclipse editpart controller for EclipseRefinedElement objects.
 * @author Dennis Wagelaar
 * @version 0.2.9, 26/04/2004
 * @since 0.2.1
 */
public class RefinedElementEditPart extends RelationElementEditPart
{
  /**
   * Handles changes in properties of this. It is
   * activated through the PropertyChangeListener.
   * It updates children, source and target connections,
   * and the visuals of this based on the property
   * changed.
   * @param evt  Event which details the property change.
   * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
   */
  public void propertyChange(PropertyChangeEvent evt)
  {
    String id = evt.getPropertyName();
    if (id.equals(Properties.ID_ICON_URI))
      refreshVisuals();
    if (Properties.ID_DEFAULT_ELEMENT_OF.equals(evt.getPropertyName()))
      refreshVisuals();
    else super.propertyChange(evt);
  }

  /**
   * @see org.eclipse.gef.GraphicalEditPart#getContentPane()
   */
  public IFigure getContentPane() 
  {
    return getConceptFigure().getContentPane();
  }

  /**
   * Installs EditPolicies specific to this.
   * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
   */
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new CoComposeXYLayoutEditPolicy());
  }

  /**
   * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
   */
  protected IFigure createFigure()
  {
    ConceptFigure fig = new ConceptFigure();
    return fig;
  }

  /**
   * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
   */
  protected void refreshVisuals() 
  {
    try
    {
      String uri = getRefinedElement().getIconUri();
      if (uri != null)
      { 
        Image icon;
        if (uri.indexOf(':') == -1)
        { // if no protocol specified, assume this is a file path
          Model model = getRefinedElement().getModel();
          Assert.isNotNull(model);
          String parent = new File(model.getUri()).getParentFile().getAbsolutePath();
          String file = new File(parent, uri).getAbsolutePath();
          icon = ImageDescriptor.createFromFile(null, file).createImage(false);
        }
        else
        { // else parse URI         
          URL url = new URL(uri);
          icon = ImageDescriptor.createFromURL(url).createImage(false);
        }
        getConceptFigure().setIcon(icon);
      }
      else
      {
        getConceptFigure().setIcon(null);
      }
    }
    catch (Exception e)
    { //should never happen
      e.printStackTrace();
    }
    getConceptFigure().setBold(
      getRefinedElement().getDefaultElementOf() != null);
    super.refreshVisuals();
  }

  /**
   * Gets the model associated with this.
   * @return the model associated with this.
   */
  protected RefinedElement getRefinedElement()
  {
    return (RefinedElement) getModel();
  }

  /**
   * Gets the figure as a ConceptFigure.
   * @return the figure as a ConceptFigure.
   */  
  protected ConceptFigure getConceptFigure()
  {
    return ((ConceptFigure) getFigure());
  }
}
