/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003 - 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.editpolicies;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.commands.ConnectionCommand;
import be.ac.vub.cocompose.eclipse.commands.CreateCommand;
import be.ac.vub.cocompose.eclipse.editparts.RelationElementEditPart;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;

/**
 * Eclipse Node edit policy for CoCompose relation elements.
 * @author Dennis Wagelaar
 * @version 0.2.8, 19/04/2004
 * @since 0.2.0
 */
public class CoComposeNodeEditPolicy extends GraphicalNodeEditPolicy
{
  /**
   * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
   */
  protected Command getConnectionCompleteCommand(CreateConnectionRequest request)
  {
    ConnectionCommand command = (ConnectionCommand) request.getStartCommand();
    command.setTargetParticipant(getRelationElement());
    ConnectionAnchor anchor = getRelationElementEditPart().getTargetConnectionAnchor(request);
    if (anchor == null)
      return null;
    CreateCommand create = new CreateCommand();
    Namespace ns = getRelationElement().getNamespace();
    while (ns.getNamespace() != null)
    { // retrieve root namespace = model
      ns = ns.getNamespace();
    }
    create.setOwner(ns);
    create.setElement(command.getRelationship());
    return command.chain(create);
  }

  /**
   * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
   */
  protected Command getConnectionCreateCommand(CreateConnectionRequest request)
  {
    ConnectionCommand command = new ConnectionCommand();
    command.setRelationship(CoCompose.getDefault().getFactory().createRelationship());
    command.setSourceParticipant(getRelationElement());
    ConnectionAnchor anchor = getRelationElementEditPart().getSourceConnectionAnchor(request);
    request.setStartCommand(command);
    return command;
  }

  /**
   * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
   */
  protected Command getReconnectTargetCommand(ReconnectRequest request)
  {
    ConnectionCommand command = new ConnectionCommand();
    command.setRelationship((Relationship) request.getConnectionEditPart().getModel());
    ConnectionAnchor anchor = getRelationElementEditPart().getTargetConnectionAnchor(request);
    command.setTargetParticipant(getRelationElement());
    return command;
  }

  /**
   * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
   */
  protected Command getReconnectSourceCommand(ReconnectRequest request)
  {
    ConnectionCommand command = new ConnectionCommand();
    command.setRelationship((Relationship) request.getConnectionEditPart().getModel());
    ConnectionAnchor anchor = getRelationElementEditPart().getSourceConnectionAnchor(request);
    command.setSourceParticipant(getRelationElement());
    return command;
  }

  /**
   * Retrieves the RelationElementEditPart for the connection.
   * @return the RelationElementEditPart.
   */
  protected RelationElementEditPart getRelationElementEditPart() 
  {
    return (RelationElementEditPart) getHost();
  }

  /**
   * Retrieves the EclipseRelationElement for the connection.
   * @return the EclipseRelationElement.
   */
  protected RelationElement getRelationElement() 
  {
    return (RelationElement) getHost().getModel();
  }
}
