/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposeTreeContainerEditPolicy.java,v 1.2 2004/04/28 12:33:20 dwagelaa Exp $
 * $Log: CoComposeTreeContainerEditPolicy.java,v $
 * Revision 1.2  2004/04/28 12:33:20  dwagelaa
 * tree drag-n-drop works
 *
 * Revision 1.1  2004/04/28 12:24:36  dwagelaa
 * worked on drag-n-drop
 * updated copy/paste
 * updated deep copy operation
 *
 */

package be.ac.vub.cocompose.eclipse.editpolicies;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editpolicies.TreeContainerEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import be.ac.vub.cocompose.eclipse.commands.CreateCommand;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;

/**
 * 
 * @author Dennis Wagelaar
 * @version , 28-apr-2004
 * @since
 */
public class CoComposeTreeContainerEditPolicy extends TreeContainerEditPolicy
{
  /**
   * Creates a CreateCommand.
   * @param element the element to insert.
   * @param parent the parent element.
   * @param index the insertion index.
   * @return the CreateCommand.
   */
  protected Command createCreateCommand(
    ModelElement element,
    Namespace parent,
    int index)
  {
    CreateCommand cmd = new CreateCommand();
    cmd.setOwner(parent);
    cmd.setElement(element);
    cmd.setIndex(index);
    cmd.setLabel("Create/Add ModelElement");
    return cmd;
  }

  /**
   * @see org.eclipse.gef.editpolicies.TreeContainerEditPolicy#getAddCommand(org.eclipse.gef.requests.ChangeBoundsRequest)
   */
  protected Command getAddCommand(ChangeBoundsRequest request)
  {
    CompoundCommand command = new CompoundCommand();
    List editparts = request.getEditParts();
    int index = findIndexOfTreeItemAt(request.getLocation());
    Namespace namespace = (Namespace) getHost().getModel();

    for (int i = 0; i < editparts.size(); i++)
    {
      EditPart child = (EditPart) editparts.get(i);
      if (isAncestor(child, getHost()))
      {
        command.add(UnexecutableCommand.INSTANCE);
      }
      else
      {
        ModelElement element = (ModelElement) child.getModel();
        command.add(createCreateCommand(element, namespace, index));
      }
    }
    return command;
  }

  /**
   * @see org.eclipse.gef.editpolicies.TreeContainerEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
   */
  protected Command getCreateCommand(CreateRequest request)
  {
    ModelElement element = (ModelElement) request.getNewObject();
    Namespace namespace = (Namespace) getHost().getModel();
    int index = findIndexOfTreeItemAt(request.getLocation());
    return createCreateCommand(element, namespace, index);
  }

  /**
   * @see org.eclipse.gef.editpolicies.TreeContainerEditPolicy#getMoveChildrenCommand(org.eclipse.gef.requests.ChangeBoundsRequest)
   */
  protected Command getMoveChildrenCommand(ChangeBoundsRequest request)
  {
    return getAddCommand(request);
  }

  /**
   * Checks whether source is an ancestor of target.
   * @param source
   * @param target
   * @return true if source is an ancestor of target.
   */
  protected boolean isAncestor(EditPart source, EditPart target)
  {
    if (source == target)
      return true;
    if (target.getParent() != null)
      return isAncestor(source, target.getParent());
    return false;
  }
}
