/*
 * Created on 25-jun-2003
 */
package be.ac.vub.cocompose.eclipse.editpolicies;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import be.ac.vub.cocompose.eclipse.commands.CreateCommand;
import be.ac.vub.cocompose.eclipse.commands.SetConstraintCommand;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RelationElement;

/**
 * Eclipse XY Layout edit policy for CoCompose elements.
 * @author Dennis Wagelaar
 * @version 0.2.0
 * @since 0.2.0
 */
public class CoComposeXYLayoutEditPolicy extends XYLayoutEditPolicy
{

  /**
   * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
   */
  protected Command createAddCommand(EditPart child, Object constraint)
  {
    CreateCommand add = new CreateCommand();
    add.setOwner((Namespace) getHost().getModel());
    add.setElement((ModelElement) child.getModel());
    add.setLabel("Add");
    return add.chain(createChangeConstraintCommand(child, constraint));
  }

  /**
   * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
   */
  protected Command createChangeConstraintCommand(
    EditPart child,
    Object constraint)
  {
    SetConstraintCommand positionCommand = new SetConstraintCommand();
    positionCommand.setElement((RelationElement) child.getModel());
    positionCommand.setPosition((Rectangle) constraint);
    return positionCommand;
  }

  /**
   * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
   */
  protected EditPolicy createChildEditPolicy(EditPart child)
  {
    return new NonResizableEditPolicy();
  }

  /**
   * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
   */
  protected Command getCreateCommand(CreateRequest request)
  {
    CreateCommand create = new CreateCommand();
    create.setOwner((Namespace) getHost().getModel());
    create.setElement((ModelElement) request.getNewObject());
    Rectangle constraint = (Rectangle) getConstraintFor(request);
    create.setPosition(constraint);
    create.setLabel("Create");
    return create;
  }

  /**
   * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
   */
  protected Command getDeleteDependantCommand(Request request)
  {
    return null;
  }
}
