/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.model;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import java.util.Collection;
import java.util.Iterator;

/** 
 * Describes the common properties for eclipse editor elements.
 * @author Dennis Wagelaar 
 * @version 0.2.0, 02/12/2003
 * @since 0.2.0
 */
public class EclipseElement implements IPropertySource
{
  private IPropertyDescriptor[] desc = null;
  private IPropertyDescriptorCreator creator = null;

  /**
   * Creates an EclipseElement.
   * @param creator the IPropertyDescriptorCreator that indicates what properties are available. May not be null!
   */
  public EclipseElement(IPropertyDescriptorCreator creator)
  {
    this.creator = creator;
  }

  /**
   * Returns an Object which represents the appropriate value for the property name supplied.
   * @param id Name of the property for which the values are needed.
   * @return  Object which is the value of the property.
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
   */
  public Object getPropertyValue(Object id)
  {
    return null;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
   */
  public boolean isPropertySet(Object id)
  {
    return true;
  }

  /**
   * Resets a property of this object.
   * @param id - the property id.
   * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
   */
  public void resetPropertyValue(Object id)
  {
  }

  /**
   * Sets the value of a given property with the value supplied.
   * Also fires a property change if necessary.
   * @param id - Name of the parameter to be changed.
   * @param value - Value to be set to the given parameter.
   * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
   */
  public void setPropertyValue(Object id, Object value)
  {
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
   */
  public Object getEditableValue()
  {
    return creator;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
   */
  public IPropertyDescriptor[] getPropertyDescriptors()
  {
    if (desc == null)
    {
      desc = createDescriptorsArray(creator.createDescriptors());
    }
    return desc;
  }

  /**
   * Creates the property descriptors array from a Collection.
   * @param desc - the descriptors collection.
   * @return the property descriptors array.
   */
  private IPropertyDescriptor[] createDescriptorsArray(Collection desc)
  {
    IPropertyDescriptor[] idesc = new IPropertyDescriptor[desc.size()];
    int index = 0;
    Iterator descIt = desc.iterator();
    while (descIt.hasNext())
    {
      idesc[index] = (IPropertyDescriptor) descIt.next();
      index++;
    }
    return idesc;
  }

  final void resetPropertyValue(String propName)
  {
  }

  final void setPropertyValue(String propName, Object val)
  {
  }
}
