/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.model.debug;

import java.net.URL;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.model.IconValidator;
import be.ac.vub.cocompose.log.Log;

/**
 * Debugging code for IconValidator.
 * @author Dennis Wagelaar
 * @version 0.2.8, 19/04/2004
 * @since 0.2.7
 * @see be.ac.vub.cocompose.eclipse.model.IconValidator
 */
public aspect IconValidatorDebug
{
  private static Log log = CoCompose.getDefault().getLog();
  
  before(String file): 
    execution(* IconValidator.imageExistsAt(String)) && 
    args(file)
  {
    beforeImageExistsAt(file);
  }
  
  before(URL url): 
    execution(* IconValidator.imageExistsAt(URL)) && 
    args(url)
  {
    beforeImageExistsAt(url.toString());
  }

  /**
   * Logs a message before imageExistsAt().
   * @param uri the URI that is checked.
   */  
  public static void beforeImageExistsAt(String uri)
  {
    log.log("icon URI = " + uri, Log.INFO, null);
    log.endLog(Log.INFO);
  }
}
