/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.io.debug;

import java.io.InputStream;
import java.io.OutputStream;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.io.ModelReader;
import be.ac.vub.cocompose.io.ModelWriter;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.log.Log;

/**
 * CoCompose2 I/O debugging code.
 * @author Dennis Wagelaar
 * @version 0.2.8, 19/04/2004
 * @since 0.2.7
 */
public aspect IODebug
{
  private static Log log = CoCompose.getDefault().getLog();

  before(String uri, ModelFactory factory):
    args(uri, factory) &&
    execution(* ModelReader.read(String, ModelFactory))
  {
    beforeRead(uri);
  }

  before(InputStream input, ModelFactory factory):
    args(input, factory) &&
    execution(* ModelReader.read(InputStream, ModelFactory))
  {
    beforeRead(input.toString());
  }
  
  before(Model model):
    args(model) &&
    execution(* ModelWriter.write(Model))
  {
    beforeWrite(model.getUri());
  }

  before(Model model, OutputStream output):
    args(model, output) &&
    execution(* ModelWriter.write(Model, OutputStream))
  {
    beforeWrite(model.getUri());
  }
  
  after():
    execution(* ModelReader.read(String, ModelFactory)) ||
    execution(* ModelReader.read(InputStream, ModelFactory)) ||
    execution(* ModelWriter.write(Model)) ||
    execution(* ModelWriter.write(Model, OutputStream))
  {
  	endLogging();
  }

  /**
   * Logs a message before a read().
   * @param source the model source location.
   */
  public static void beforeRead(String source)
  {
    log.log("Reading model from " + source, Log.INFO, null);
  }

  /**
   * Logs a message before a write().
   * @param source the model source location.
   */
  public static void beforeWrite(String source)
  {
    log.log("Writing model to " + source, Log.INFO, null);
  }

  /**
   * Ends logging.
   */
  public static void endLogging()
  {
    log.endLog(Log.INFO);
  }
}
