/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIElementTypeSwitch.java,v 1.3 2004/04/28 12:23:29 dwagelaa Exp $
 * $Log: XMIElementTypeSwitch.java,v $
 * Revision 1.3  2004/04/28 12:23:29  dwagelaa
 * + added parsing of ImplementationPattern.content tag
 *
 */

package be.ac.vub.cocompose.io.xmi;

import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.ModelElement;

/**
 * Determines the CoCompose element type and returns a new instance of that type
 * @author Dennis Wagelaar
 * @version 0.2.0, 08/01/2004
 * @since 0.2.0
 */
public class XMIElementTypeSwitch
{
  private ModelFactory factory;

  public XMIElementTypeSwitch(ModelFactory factory) throws ModelElementException
  {
    if (factory == null)
      throw new ModelElementException("Model factory may not be null");
    this.factory = factory;
  }
  
  /**
   * Creates a CoCompose element of the given type.
   * @param type the element type
   * @return the created element, or null if the type is not supported
   */
  public ModelElement createElement(String type)
  {
    if (type.equals(XMI.CoCompose.qName("Model")))
      return factory.createModel();
    else if (type.equals(XMI.CoCompose.qName("Concept")))
      return factory.createConcept();
    else if (type.equals(XMI.CoCompose.qName("Role")))
      return factory.createRole();
    else if (type.equals(XMI.CoCompose.qName("Relationship")))
      return factory.createRelationship();
    else if (type.equals(XMI.CoCompose.qName("SolutionPattern")))
      return factory.createSolutionPattern();
    else if (type.equals(XMI.CoCompose.qName("ImplementationPattern")))
      return factory.createImplementationPattern();
    else if (type.equals(XMI.CoCompose.qName("ImplementationGenerator")))
      return factory.createImplementationGenerator();
    else if (type.equals(XMI.CoCompose.qName("ImplementationRolePart")))
      return factory.createImplementationRolePart();
    else if (type.equals(XMI.CoCompose.qName("SimpleProperty")))
      return factory.createSimpleProperty();
    else if (type.equals(XMI.CoCompose.qName("ModelProperty")))
      return factory.createModelProperty();
    else if (type.equals(XMI.CoCompose.qName("ElementConstraint")))
      return factory.createElementConstraint();
    else if (type.equals(XMI.CoCompose.qName("AndConstraint")))
      return factory.createAndConstraint();
    else if (type.equals(XMI.CoCompose.qName("OrConstraint")))
      return factory.createOrConstraint();
    else
      return null;
  }
}
