/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: SAXDebug.java,v 1.2 2004/04/28 12:23:29 dwagelaa Exp $
 * $Log: SAXDebug.java,v $
 * Revision 1.2  2004/04/28 12:23:29  dwagelaa
 * + added parsing of ImplementationPattern.content tag
 *
 */

package be.ac.vub.cocompose.io.xml.debug;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.log.Log;

/**
 * SAX XML parsing and writing debugging code.
 * @author Dennis Wagelaar
 * @version 0.2.8, 19/04/2004
 * @since 0.2.7
 */
public aspect SAXDebug
{
  private static Log log = CoCompose.getDefault().getLog();
  
  before(String uri, String local, String raw, Attributes attrs):
	args(uri, local, raw, attrs) &&
    execution(* ContentHandler.startElement(String, String, String, Attributes))
  {
    beforeStartElement(uri, local, raw, attrs);
  }
  
  before(String uri, String local, String raw):
    args(uri, local, raw) &&
    execution(* ContentHandler.endElement(String, String, String))
  {
    beforeEndElement(uri, local, raw);
  }
  
  /**
   * Logs a message before a startElement().
   * @param msg The message to log.
   */
  public static void beforeStartElement(String uri, String local, String raw, 
    Attributes attrs)
  {
    log.log("start " + uri + ":" + local + " (" + raw + ")", Log.INFO, null);
  }

  /**
   * Logs a message before an endElement().
   * @param msg The message to log.
   */
  public static void beforeEndElement(String uri, String local, String raw)
  {
    log.endLog(Log.INFO);
    log.log("end " + uri + ":" + local + " (" + raw + ")", Log.INFO, null);
    log.endLog(Log.INFO);
  }
}
