/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang;

import java.util.List;
import java.util.Vector;

import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.Role;

/**
 * Default implementation for visiting a Model. Propagates every element's visit through to its superclass visits and traverses its composition relationships.
 * @author Dennis Wagelaar 
 * @version 0.2.3, 19/01/2004
 * @since 0.2.0
 */
abstract public class DefaultTraversingModelVisitor extends DefaultSuperModelVisitor
{
  /**
   * Visits a Namespace.
   * @param namespace - the element to visit.
   * @exception ModelElementException
   */
  public void visitNamespace(Namespace namespace) throws ModelElementException
  {
    super.visitNamespace(namespace);
    // Use copied list, since elements may be deleted during traversal
    List ownedElts = new Vector(namespace.getOwnedElements());
    for (int i = 0; i < ownedElts.size(); i++)
    {
      ((ModelElement) ownedElts.get(i)).accept(current);
    }
  }

  /**
   * Visits a Role.
   * @param role - the element to visit.
   * @exception ModelElementException
   */
  public void visitRole(Role role) throws ModelElementException
  {
    super.visitRole(role);
    role.getMultiplicity().accept(current);
  }
}
