/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang;

import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.constraints.AndConstraint;
import be.ac.vub.cocompose.lang.constraints.OrConstraint;
import be.ac.vub.cocompose.lang.properties.SimpleProperty;
import be.ac.vub.cocompose.lang.properties.ModelProperty;

/**
 * Abstract factory interface for creating model elements.
 * @author Dennis Wagelaar 
 * @version 0.2.0, 09/01/2004
 * @since 0.2.0
 */
public interface ModelFactory
{
  /**
   * Creates a Model.
   * @return the Model.
   */
  Model createModel();

  /**
   * Creates a Relationship.
   * @return the Relationship.
   */
  Relationship createRelationship();

  /**
   * Creates a SolutionPattern.
   * @return the SolutionPattern.
   */
  SolutionPattern createSolutionPattern();

  /**
   * Creates an ImplementationGenerator.
   * @return the ImplementationGenerator.
   */
  ImplementationGenerator createImplementationGenerator();

  /**
   * Creates an ImplementationRolePart.
   * @return the ImplementationRolePart.
   */
  ImplementationRolePart createImplementationRolePart();

  /**
   * Creates an ImplementationPattern.
   * @return the ImplementationPattern.
   */
  ImplementationPattern createImplementationPattern();

  /**
   * Creates a Concept.
   * @return the Concept.
   */
  Concept createConcept();

  /**
   * Creates a Role.
   * @return the Role.*/
  Role createRole();

  /**
   * Creates an ElementConstraint.
   * @return the ElementConstraint.*/
  ElementConstraint createElementConstraint();

  /**
   * Creates an AndConstraint.
   * @return the AndConstraint.*/
  AndConstraint createAndConstraint();

  /**
   * Creates an OrConstraint.
   * @return the OrConstraint.*/
  OrConstraint createOrConstraint();

  /**
   * Creates a SimpleProperty.
   * @return the SimpleProperty.*/
  SimpleProperty createSimpleProperty();

  /**
   * Creates a ModelProperty.
   * @return the ModelProperty.*/
  ModelProperty createModelProperty();
}
