package be.ac.vub.cocompose.lang.constraints;

import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.Namespace;

/**
 * General constraint interface.
 * @author Dennis Wagelaar 
 * @version 0.2.0
 * @since 0.0.23
 */
public interface Constraint extends Namespace
{
  /**
   * Checks whether the constraint holds.
   * @exception ModelElementException - if the hard constraint doesn't hold.
   */
  void check() throws ModelElementException ;

  /**
   * Gets the owner constraint.
   * @return the owner constraint.
   */
  CompositeConstraint getOwnerConstraint();

  /**
   * Sets the owner constraint.
   * @param ownerConstraint - the owner constraint.
   */
  void setOwnerConstraint(CompositeConstraint ownerConstraint);

  /**
   * Gets the inverted attribute.
   * @return true if and only if the constraint is inverted.
   */
  boolean isInverted();

  /**
   * Sets the inverted attribute.
   * @param inverted - if true, the constraint is inverted.
   */
  void setInverted(boolean inverted);
}
