/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ModelElement.java,v 1.3 2004/04/27 16:43:58 dwagelaa Exp $
 * $Log: ModelElement.java,v $
 * Revision 1.3  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.lang.core;

import be.ac.vub.cocompose.lang.Element;
import be.ac.vub.cocompose.lang.VisitedElement;
import be.ac.vub.cocompose.lang.ModelElementException;

/**
 * Root interface for each CoCompose language element.
 * @version 0.2.9, 27/04/2004
 * @since 0.2.0
 * @author Dennis Wagelaar 
 */
public interface ModelElement extends Element, Cloneable, VisitedElement
{
  /**
   * Gets the name of this ModelElement.
   * @return the name of this ModelElement.
   */
  public String getName();

  /**
   * Sets the name of this ModelElement.
   * @param name - the name of this ModelElement.
   * @exception ModelElementException - if the name is already used within the namespace.
   */
  public void setName(String name) throws ModelElementException;

  /**
   * Gets the namespace that contains this element.
   * @return the namespace.
   */
  public Namespace getNamespace();

  /**
   * Sets the namespace that contains this element.
   * @param namespace - the namespace.
   */
  public void setNamespace(Namespace namespace);

  /**
   * Gets the Id of this element, based upon its namespace.
   * @return the Id, or null if the namespace is not set.
   */
  public String getId();

  /**
   * Sets the Id of this element.
   * @param id - the Id.
   */
  public void setId(String id);

  /**
   * Gets the root namespace for this element, if any.
   * @return the root namespace, if any.
   */
  public Namespace getRootNamespace();

  /**
   * Gets the root model for this element, if any.
   * @return the root model, if any.
   */
  public Model getModel();
}
