/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003 - 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang.impl.constraints;

import java.util.Iterator;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.constraints.OrConstraint;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.ModelVisitor;

/**
 * Boolean "or" composite constraint interface.
 * @author Dennis Wagelaar
 * @version 0.2.0
 * @since 0.2.0
 */
public class DefaultOrConstraint extends DefaultCompositeConstraint implements OrConstraint
{
  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.*/
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitOrConstraint(this);
  }

  /**
   * Invokes superclass accept().
   * @param modelVisitor The visitor to accept.
   * @see #accept(ModelVisitor)*/
  public void acceptSuper(ModelVisitor modelVisitor) throws ModelElementException
  {
    super.accept(modelVisitor);
  }

  /**
   * Checks whether the constraint holds.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  protected void concreteCheck() throws ModelElementException
  {
    ModelElementException lastException = null;
    Iterator ownedConstr = getOwnedConstraints().iterator();
    while (ownedConstr.hasNext())
    {
      try
      {
        ((Constraint) ownedConstr.next()).check();
        return;
      }
      catch (ModelElementException e)
      {
      }
    }
    if (lastException != null)
      throw lastException;
  }
}
